local _G = getfenv(0)
local ADDON_NAME, addon = ...

local LSM = _G.LibStub:GetLibrary("LibSharedMedia-3.0")

local Bar = {}

-- Table which has pointers to all the bars.
addon.bars = {}

-- Define a generic class for the bars
Bar.__index = Bar
addon.Bar = Bar

function Bar:Create(settings)
	-- Old params: name, fontAdj, width, height, setPoint, setValuePoint
	local object = _G.setmetatable({}, Bar)
	object.name = settings.name
	object.fontAdj = settings.fontAdj
	
	if settings.width and _G.type(settings.width) ~= "function" then
		object.GetWidth = function() return settings.width end
	end

	if settings.height and _G.type(settings.height) ~= "function" then
		object.GetHeight = function() return settings.height end
	end

	if settings.functions then
		for name, func in _G.pairs(settings.functions) do
			if func and _G.type(func) == "function" then
				object[name] = func
			end
		end
	end

	if not object.SetDesiredPoint then
		object.SetDesiredPoint = object.SetPoint
	end

	object:Initialize(settings)
	-- Add the bar to the addon's table of bars
	addon.bars[settings.name] = object
	return object
end

function Bar:Initialize(settings)
	local bar = _G.CreateFrame("StatusBar", 
		addon.addonNameCondensed .. "_"  .. self.name, addon.frames.main)
	self.bar = bar
	bar.parent = self
	self.db = addon.db.profile.bars[self.name]
	self.altcolor = false
	self.isRuneBar = self.name:find("RuneBar%d")
	bar:SetScale(1)
	local orientation = "HORIZONTAL"
	if self.GetOrientation and _G.type(self.GetOrientation) == "function" then
		orientation = self:GetOrientation()
	end
	bar:SetOrientation(orientation)
	bar:SetWidth(self:GetWidth())
	bar:SetHeight(self:GetHeight())
	local bt = LSM:Fetch("statusbar", addon.db.profile.texture)
	bar:SetStatusBarTexture(bt)
	bar:GetStatusBarTexture():SetHorizTile(false)
	bar:GetStatusBarTexture():SetVertTile(false)
	if not self.isRuneBar then
		local bc = self.db.color
		bar:SetStatusBarColor(bc.r, bc.g, bc.b, bc.a)
	end
	bar.bg = bar:CreateTexture(nil, "BACKGROUND")
	bar.bg:SetTexture(bt)
	bar.bg:SetAllPoints(true)
	if not self.isRuneBar then
		local bgc = self.db.bgcolor
		bar.bg:SetVertexColor(bgc.r, bgc.g, bgc.b, bgc.a)
	end

	local ff, fh, fflags = addon.GetFontSettings()
	local font = LSM:Fetch("font", ff)
	bar.value = bar:CreateFontString(nil, "OVERLAY")
	bar.value:SetPoint("CENTER")
	bar.value:SetFont(font, fh + self.fontAdj, fflags)
	bar.value:SetJustifyH("CENTER")
	--bar.value:SetShadowOffset(1, -1)
	local tc = self.db.textcolor
	bar.value:SetTextColor(tc.r, tc.g, tc.b, tc.a)
	if self.isRuneBar and addon.db.profile.rechargeTimers == "None" then
		bar.value:Hide()
	end
	if settings.setValueToZero then
		bar.value:SetText("0")
	end

	if settings.hasRechargeTime then
		bar.rechargeTime = bar:CreateFontString(nil, "OVERLAY")
		bar.rechargeTime:SetPoint("CENTER")
		bar.rechargeTime:SetFont(font, fh + self.fontAdj - 1, fflags)
		bar.rechargeTime:SetJustifyH("CENTER")
		local tc = self.db.rechargeTextColor
		bar.rechargeTime:SetTextColor(tc.r, tc.g, tc.b, tc.a)
		bar.rechargeTime:SetText("0")
		bar.rechargeTime:Hide()
		bar.recharge = 0
	end

	if self.SetPoint then self:SetPoint() end
	if self.SetValuePoint then
		self:SetValuePoint()
	else
	    bar.value:SetPoint("CENTER")
	end

	if settings.initTimer then
		bar.active = false
		bar.timer = 0
		bar:Hide()
	end

	bar.locked = true
end

function Bar:Show()
	self.bar:Show()
end

function Bar:Hide()
	self.bar:Hide()
end

function Bar:IsLocked()
	return self.bar.locked
end

function Bar:Lock(locked)
	if locked ~= nil then
		self.bar.locked = locked
	end
	self.bar:EnableMouse(not self.bar.locked)
	if self.name == "BloodShieldBar" then
		if not self.bar.locked then
			self.bar:Show()
		elseif not self.bar.active then
			self.bar:Hide()
		end
	end
end

function Bar:SetMovable()
	local bar = self.bar
    bar:SetMovable(true)
    bar:RegisterForDrag("LeftButton")
    bar:SetScript("OnDragStart",
        function(self, button)
			if not self.locked then
            	self:StartMoving()
			end
        end)
    bar:SetScript("OnDragStop",
        function(self)
            self:StopMovingOrSizing()
			local scale = self:GetEffectiveScale() / _G.UIParent:GetEffectiveScale()
			local x, y = self:GetCenter()
			x, y = x * scale, y * scale
			x = x - _G.GetScreenWidth()/2
			y = y - _G.GetScreenHeight()/2
			x = x / self:GetScale()
			y = y / self:GetScale()
			self.parent.db.x, self.parent.db.y = x, y
			self:SetUserPlaced(false);
        end)
	self:Lock()
end

local numberFmt = "%.0f"
function Bar:SetValue(value)
	self.bar:SetValue(value)
	self.bar.value:SetText(numberFmt:format(value))
end

function Bar:SetValueTextColor(color)
   	local tc = color or self.db.textcolor
   	self.bar.value:SetTextColor(tc.r, tc.g, tc.b, tc.a)
end

function Bar:Reset()
	self:ResetFonts()
	self:UpdateVisibility()
	--self:UpdateTexture()
	self:UpdateGraphics()
end

function Bar:ResetFonts()
	local ff, fh, fontFlags = addon.GetFontSettings()
	local font = LSM:Fetch("font", ff)
	self.bar.value:SetFont(font, fh + self.fontAdj, fontFlags)
	self.bar.value:SetText(self.bar.value:GetText())
end

function Bar:UpdateVisibility()
	if self.GetWidth then self.bar:SetWidth(self:GetWidth()) end
	if self.GetHeight then self.bar:SetHeight(self:GetHeight())	end
	if self.SetPoint and not self.isRuneBar then self:SetPoint() end
	if self.SetValuePoint then
		self:SetValuePoint()
	else
		self.bar.value:SetPoint("CENTER")
	end
	if not self.db.enabled and not self.isRuneBar then
		self.bar:Hide()
		self.bar:SetScript("OnUpdate", nil)
	end

	self:UpdateTexture()
end

function Bar:UpdateTexture()
	local bt = LSM:Fetch("statusbar", addon.db.profile.texture)
	self.bar:SetStatusBarTexture(bt)
	self.bar.bg:SetTexture(bt)
	self.bar:GetStatusBarTexture():SetHorizTile(false)
	self.bar:GetStatusBarTexture():SetVertTile(false)
	--self:UpdateGraphics()
end

function Bar:UpdateGraphics()
	if not self.isRuneBar then
		local bc, bgc, tc
		if self.altcolor then
			bc = self.db.alt_color or self.db.color
			bgc = self.db.alt_bgcolor or self.db.bgcolor
			tc = self.db.alt_textcolor or self.db.textcolor
		else
			bc = self.db.color
			bgc = self.db.bgcolor
			tc = self.db.textcolor
		end

		self.bar:SetStatusBarColor(bc.r, bc.g, bc.b, bc.a)
		self.bar.bg:SetVertexColor(bgc.r, bgc.g, bgc.b, bgc.a)
		self.bar.value:SetTextColor(tc.r, tc.g, tc.b, tc.a)
	end
end
